@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2018
+  Copyright (c) 2006 - 2020
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -30,6 +30,7 @@
 #include <iostream>
 #include <streambuf>
 
+/// \brief Specialization of std::streambuf for output to a Unix file descriptor
 class fdoutbuf: public std::streambuf {
  public:
   explicit fdoutbuf( int _fd );
@@ -37,19 +38,28 @@ class fdoutbuf: public std::streambuf {
   bool connect( int );
  protected:
   virtual int overflow( int );
-  virtual std::streamsize xsputn( const char *, std::streamsize );
+  virtual std::streamsize xsputn( const char *s, std::streamsize num );
   int fd; // file descriptor
 };
 
+/// \brief An output stream connected to a Unix file descriptor
 class fdostream: public std::ostream {
  protected:
   fdoutbuf buf;
  public:
- explicit fdostream( int fd ): std::ostream(&buf), buf(fd) {};
- fdostream(): std::ostream(&buf) {};
+ explicit fdostream( int fd ): std::ostream(&buf), buf(fd) {
+    /// create an fd outputstream
+    /*!
+      \param fd the file descriptor to use
+    */
+  };
+ fdostream(): std::ostream(&buf) {
+    /// create an fd outputstream
+  };
   bool open( int fd );
 };
 
+/// \brief Specialization of std::streambuf for input from a Unix file descriptor
 class fdinbuf: public std::streambuf {
  public:
   fdinbuf();
@@ -63,15 +73,23 @@ class fdinbuf: public std::streambuf {
   char buffer[bufferSize];
 };
 
+/// \brief An input stream connected to a Unix file descriptor
 class fdistream: public std::istream {
  protected:
   fdinbuf buf;
  public:
-  explicit fdistream( int fd ): std::istream(&buf), buf(fd) {};
- fdistream(): std::istream(&buf) {};
+  explicit fdistream( int fd ): std::istream(&buf), buf(fd) {
+    /// create an fd inputstream
+    /*!
+      \param fd the file descriptor to use
+    */
+  };
+ fdistream(): std::istream(&buf) {
+    /// create an fd inputstream
+  };
   bool open( int fd );
 };
 
-bool nb_getline( std::istream&, std::string&, int& );
-bool nb_putline( std::ostream&, const std::string&, int& );
+bool nb_getline( std::istream& is, std::string& result, int& timeout );
+bool nb_putline( std::ostream& os, const std::string& what, int& timeout );
 #endif
