@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2018
+  Copyright (c) 2006 - 2020
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -30,44 +30,14 @@
 #include <string>
 #include "ticcutils/LogBuffer.h"
 
-#if defined __GNUC__ || __IBMCPP__
-#define LTGT <>
-#else
-#define LTGT
-#endif
-
 namespace TiCC {
 
-  template<class T> class o_manip;
-  template <class T> std::ostream& operator << (std::ostream&, const o_manip<T>& );
-
-  template<class T> class o_manip{
-    typedef std::ostream& (*FRM)(std::ostream&, T );
-    FRM func;
-    T l;
-  public:
-    o_manip<T>( FRM f, T ll ): func(f), l(ll) {};
-    friend std::ostream& operator << LTGT (std::ostream&, const o_manip<T>& );
-    o_manip<T>( const o_manip<T>& o ):
-    func(o.func), l(o.l){};
-  private:
-    o_manip<T> operator=( const o_manip<T>& );
-  };
-
-  template<class T>
-    inline std::ostream& operator << (std::ostream& os,
-				      const o_manip<T>& m ){
-    return m.func( os, m.l );
-  }
-
+  /// \brief Logstream is a class to synchronize output in Multi-Threaded
+  /// programs.
+  ///
+  /// LogStream used mutexes to assure that output from different threads is
+  /// not mangled
   class LogStream : public std::ostream {
-    friend o_manip<LogLevel> setlevel( LogLevel );
-    friend o_manip<LogLevel> setthreshold( LogLevel );
-    friend o_manip<LogFlag> setstamp( LogFlag );
-    friend o_manip<const std::string& > setmessage( const std::string& );
-    friend o_manip<const std::string&> addmessage( const std::string& );
-    friend o_manip<const std::string&> addmessage( const int );
-    friend o_manip<const std::string&> write_buf(const std::string& );
     friend bool IsActive( LogStream & );
     friend bool IsActive( LogStream * );
   public:
@@ -105,6 +75,7 @@ namespace TiCC {
   bool IsActive( LogStream & );
   bool IsActive( LogStream * );
 
+  /// \brief create a LogStream
   class Log{
   public:
     explicit Log( LogStream * );
@@ -118,6 +89,7 @@ namespace TiCC {
     Log& operator=( const Log& );
   };
 
+  /// \brief create a debugging LogStream
   class Dbg{
   public:
     explicit Dbg( LogStream * );
@@ -131,6 +103,7 @@ namespace TiCC {
     Dbg& operator=( const Dbg& );
   };
 
+  /// \brief a debugging LogStream for heavy output
   class xDbg{
   public:
     explicit xDbg( LogStream * );
@@ -144,6 +117,7 @@ namespace TiCC {
     xDbg& operator=( const xDbg& );
   };
 
+  /// \brief a debugging LogStream for extreme output
   class xxDbg{
   public:
     explicit xxDbg( LogStream * );
