@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2018
+  Copyright (c) 2006 - 2020
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -31,16 +31,23 @@
 #include <iostream>
 
 namespace TiCC {
+
+  /// a class to keep Time in a practical way
   class Timer {
   public:
     friend std::ostream& operator << ( std::ostream& os, const Timer& T );
     Timer(){ reset(); };
-    void reset(){ myTime.tv_sec=0; myTime.tv_usec=0; };
+    void reset(){
+      /// set the Timer to sll 0
+      myTime.tv_sec=0; myTime.tv_usec=0;
+    };
     void start(){
+      /// start the Timer
       gettimeofday( &startTime, 0 );
     };
     void stop();
-    unsigned int secs() { stop(); return myTime.tv_sec; };
+    Timer& operator+=( const Timer& );
+    friend Timer operator+( Timer, const Timer& );
     std::string toString();
     static std::string now();
   private:
