@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2018
+  Copyright (c) 2006 - 2020
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -30,6 +30,7 @@
 #include <string>
 #include <map>
 #include <list>
+#include <map>
 #include "libxml/tree.h"
 
 namespace TiCC {
@@ -108,6 +109,18 @@ namespace TiCC {
     return "";
   }
 
+  inline std::map<std::string,std::string> getAttributes( const xmlNode *node ){
+    std::map<std::string,std::string> result;
+    if ( node ){
+      xmlAttr *a = node->properties;
+      while ( a ){
+	result[(char*)a->name ] = (char *)a->children->content;
+	a = a->next;
+      }
+    }
+    return result;
+  }
+
   std::string getNS( const xmlNode *, std::string& );
   inline std::string getNS( const xmlNode *n ) {
     std::string s;
@@ -139,6 +152,7 @@ namespace TiCC {
     return result;
   }
 
+  /// \brief XmlDoc is a C++ wrapper around libxml2::xmlDoc
   class XmlDoc {
     friend std::ostream& operator << ( std::ostream& , const XmlDoc& );
   public:
@@ -171,10 +185,10 @@ namespace TiCC {
     return os;
   }
 
-  std::list<xmlNode*> FindNodes( xmlNode *, const std::string& );
-  xmlNode *xPath( xmlNode *, const std::string& );
-  std::list<xmlNode*> FindNodes( xmlDoc *, const std::string& );
-  xmlNode *xPath( xmlDoc *, const std::string& );
+  std::list<xmlNode*> FindNodes( const xmlNode *, const std::string& );
+  xmlNode *xPath( const xmlNode *, const std::string& );
+  std::list<xmlNode*> FindNodes( const xmlDoc *, const std::string& );
+  xmlNode *xPath( const xmlDoc *, const std::string& );
 
 } // namespace TiCC
 
