@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2018
+  Copyright (c) 2006 - 2020
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -42,26 +42,33 @@
 using namespace std;
 using namespace TiCC;
 
-#define LOG *Log(myLog)
+#define LOG *Log(_my_log)
 
-namespace TimblServer {
+namespace TiCCServer {
 
-  const string serv_short_opts = "S:C:";
-  const string serv_long_opts =
-    // leave the , below!. This string is appended later
-    ",pidfile:,logfile:,daemonize::,debug:,config:";
+  const string serv_short_opts = "S:C:c:";
+  const string serv_long_opts = ",pidfile:,logfile:,daemonize:,debug:,config:,protocol:";
+    // leave the , in front of this line!. This string is appended later.
 
   string Version() { return VERSION; }
   string VersionName() { return PACKAGE_STRING; }
 
-  childArgs::childArgs( ServerBase *server, Sockets::ServerSocket *sock ):
+  childArgs::childArgs( ServerBase *server, Sockets::ClientSocket *sock ):
     _mother(server),_socket(sock){
+    /// create a childArgs structure
+    /*!
+      \param server our Server object
+      \param sock the Socket object
+
+      This fuction opens an input and an output stream connected to the socket.
+    */
     _id = _socket->getSockId();
     _is.open(_id);
     _os.open(_id);
   }
 
   childArgs::~childArgs( ){
+    /// destroy the childArgs object
     _os.flush();
     delete _socket;
   }
@@ -70,24 +77,32 @@ namespace TimblServer {
     string result;
     ostringstream oss;
     oss << VERSION;
-    if ( full )
+    if ( full ){
       oss << ", compiled on " << __DATE__ << ", " << __TIME__;
+    }
     result = oss.str();
     return result;
   }
 
-  ServerBase::ServerBase( const Configuration *c ):
-    myLog("BasicServer"),
-    doDaemon( true ),
-    debug( false ),
-    _maxConn( 25 ),
-    serverPort( 7000 ),
-    callback_data( 0 ),
-    config(c)
+  ServerBase::ServerBase( const Configuration *config,
+			  void *callback_data ):
+    _my_log("BasicServer"),
+    _do_daemon( true ),
+    _debug( false ),
+    _max_conn( 25 ),
+    _server_port( 7000 ),
+    _callback_data( callback_data ),
+    _protocol( "tcp" ),
+    _config(config)
   {
-    string value = config->lookUp( "port" );
+    /// create a Basic Server
+    /*!
+      \param config the configuration informatio to use
+      \param callback_data a structure with data to use in every call
+    */
+    string value = _config->lookUp( "port" );
     if ( !value.empty() ){
-      if ( !stringTo( value, serverPort ) ){
+      if ( !stringTo( value, _server_port ) ){
 	string mess = "ServerBase: invalid value '" + value + "' for port";
 	throw runtime_error( mess );
       }
@@ -96,63 +111,87 @@ namespace TimblServer {
       string mess = "ServerBase:missing 'port' in config ";
       throw runtime_error( mess );
     }
-    value = config->lookUp( "maxconn" );
+    value = _config->lookUp( "maxconn" );
     if ( !value.empty() ){
-      if ( !stringTo( value, _maxConn ) ){
+      if ( !stringTo( value, _max_conn ) ){
 	string mess = "ServerBase: invalid value '" + value + "' for maxconn";
 	throw runtime_error( mess );
       }
     }
-    value = config->lookUp( "protocol" );
+    value = _config->lookUp( "protocol" );
     if ( !value.empty() ){
-      serverProtocol = value;
+      _protocol = value;
     }
-    else {
-      serverProtocol = "tcp";
-    }
-    value = config->lookUp( "daemonize" );
+    value = _config->lookUp( "daemonize" );
     if ( !value. empty() ){
-      if ( value == "no" )
-	doDaemon = false;
-      else if ( value == "yes" )
-	doDaemon = true;
+      if ( value == "no" ){
+	_do_daemon = false;
+      }
+      else if ( value == "yes" ){
+	_do_daemon = true;
+      }
       else {
 	string mess = "ServerBase: invalid value '" + value
 	  + "' for --daemonize";
 	throw runtime_error( mess );
       }
     }
-    value = config->lookUp( "logfile" );
-    if ( !value.empty() )
-      logFile = value;
-    value = config->lookUp( "pidfile" );
-    if ( !value.empty() )
-      pidFile = value;
-    value = config->lookUp( "name" );
-    if ( !value.empty() )
-      name = value;
-    else
-      name = serverProtocol + "-server";
-    myLog.message( name );
-    value = config->lookUp( "debug" );
+    value = _config->lookUp( "logfile" );
     if ( !value.empty() ){
-      if ( value == "no" )
-	debug = false;
-      else if ( value == "yes" )
-	debug = true;
+      _log_file = value;
+    }
+    value = _config->lookUp( "pidfile" );
+    if ( !value.empty() ){
+      _pid_file = value;
+    }
+    value = _config->lookUp( "name" );
+    if ( !value.empty() ){
+      _name = value;
+    }
+    else {
+      _name = _protocol + "-server";
+    }
+    _my_log.message( _name );
+    value = _config->lookUp( "debug" );
+    if ( !value.empty() ){
+      if ( value == "no" ){
+	_debug = false;
+      }
+      else if ( value == "yes" ){
+	_debug = true;
+      }
       else {
 	string mess = "ServerBase: invalid value '" + value + "' for --debug";
 	throw runtime_error( mess );
       }
     }
-    tcp_socket = 0;
+    _socket = 0;
+  }
+
+  void ServerBase::server_usage(void) {
+    /// display helpful information
+    cerr << "Server options" << endl;
+    cerr << "--config=<f> or -c <f> : read server settings from file <f>" << endl;
+    cerr << "--pidfile=<f> : store pid in file <f>" << endl;
+    cerr << "--logfile=<f> : log server activity in file <f>" << endl;
+    cerr << "--daemonize=[yes|no] (default yes)" << endl;
+    cerr << "--protocol=[tcp|http|json] (default tcp)" << endl << endl;
+    cerr << "OR, without config file:" << endl;
+    cerr << "-S <port> : run as a server on <port>" << endl;
+    cerr << "-C <num>  : accept a maximum of 'num' parallel connections (default 10)" << endl;
   }
 
   Configuration *initServerConfig( TiCC::CL_Options& opts ){
+    /// initialize a Configuration from command-line options
+    /*!
+      \param opts The command line options
+      \return the created Configuration
+    */
     Configuration *config = new Configuration();
     bool old = false;
     string value;
-    if ( !opts.extract( "config", value ) ){
+    if ( !opts.extract( "config", value )
+	 && !opts.extract( 'c', value ) ){
       if ( opts.extract( 'S', value ) ){
 	config->setatt( "port", value );
 	old = true;
@@ -161,7 +200,7 @@ namespace TimblServer {
 	}
       }
       if ( !old ){
-	cerr << "missing --config option" << endl;
+	cerr << "missing '--config' or '-c' option" << endl;
 	delete config;
 	return 0;
       }
@@ -178,13 +217,17 @@ namespace TimblServer {
       config->setatt( "logfile", value );
     }
     if ( opts.extract( "daemonize", value ) ){
-      if ( value.empty() )
+      if ( value.empty() ){
 	value = "true";
+      }
       config->setatt( "daemonize", value );
     }
     if ( opts.extract( "debug", value ) ){
       config->setatt( "debug", value );
     }
+    if ( opts.extract( "protocol", value ) ){
+      config->setatt( "protocol", value );
+    }
     if ( old ){
       string rest = opts.toString();
       config->setatt( "default", rest );
@@ -192,47 +235,9 @@ namespace TimblServer {
     return config;
   }
 
-  string getProtocol( const string& serverConfigFile ){
-    string result = "tcp";
-    ifstream is( serverConfigFile );
-    if ( !is ){
-      cerr << "problem reading " << serverConfigFile << endl;
-      return result;
-    }
-    else {
-      string line;
-      while ( getline( is, line ) ){
-	if ( line.empty() || line[0] == '#' )
-	  continue;
-	string::size_type ispos = line.find('=');
-	if ( ispos == string::npos ){
-	  cerr << "invalid entry in: " << serverConfigFile
-	       << "offending line: '" << line << "'" << endl;
-	  return result;
-	}
-	else {
-	  string base = line.substr(0,ispos);
-	  string rest = line.substr( ispos+1 );
-	  if ( !rest.empty() ){
-	    string tmp = base;
-	    lowercase(tmp);
-	    if ( tmp == "protocol" ){
-	      string protocol = rest;
-	      lowercase( protocol );
-	      if ( protocol != "http" && protocol != "tcp" ){
-		cerr << "invalid protocol: " << protocol << endl;
-		return result;
-	      }
-	      return protocol;
-	    }
-	  }
-	}
-      }
-    }
-    return result;
-  }
-
   void *ServerBase::callChild( void *a ) {
+    /// generic callback function
+    /// pass the argument as a childArgs struct to the Server
     childArgs* ca = reinterpret_cast<childArgs*>(a);
     ca->mother()->socketChild( ca );
     return 0;
@@ -241,6 +246,7 @@ namespace TimblServer {
   static bool keepGoing = true;
 
   void KillServerFun( int Signal ){
+    /// function to handle SIGTERM signals
     if ( Signal == SIGTERM ){
       cerr << "KillServerFun caught a signal SIGTERM" << endl;
       keepGoing = false; // so stop accepting new connections
@@ -250,6 +256,7 @@ namespace TimblServer {
   }
 
   void AfterDaemonFun( int Signal ){
+    /// function to handle SIGCHILD signals
     cerr << "AfterDaemonFun caught a signal " << Signal << endl;
     if ( Signal == SIGCHLD ){
       exit(1);
@@ -257,13 +264,19 @@ namespace TimblServer {
   }
 
   void BrokenPipeChildFun( int Signal ){
+    /// function to handle SIGPIPE signals
     cerr << "BrokenPipeChildFun caught a signal " << Signal << endl;
     if ( Signal == SIGPIPE ){
       signal( SIGPIPE, BrokenPipeChildFun );
     }
   }
 
-
+  /// run the Server as a daemon
+  /*!
+    \param noCD if ≠ 0, don't cd to the root dir
+    \param noClose if ≠ 0 don't close stdin and stdout
+    \return
+  */
 #ifdef HAVE_DAEMON
   int ServerBase::daemonize( int noCD , int noClose ){
     return daemon( noCD, noClose );
@@ -315,15 +328,23 @@ namespace TimblServer {
 #endif // HAVE_DAEMON
 
   void ServerBase::sendReject( ostream& os ) const {
+    /// send message that we are too busy
     os << "Maximum connections exceeded." << endl;
     os << "try again later..." << endl;
   }
 
-  // ***** This is the routine that is executed from a new TCP thread *******
   void ServerBase::socketChild( childArgs *args ){
+    /// run a child in a new thread
+    /*!
+      \param args the arguments to use.
+
+      this function calls callback() and will run until that function
+      returns.
+    */
     signal( SIGPIPE, BrokenPipeChildFun );
     LOG << "Thread " << (uintptr_t)pthread_self() << " on socket "
-	<< args->id() << ", started at: " << Timer::now() << endl;
+	<< args->id() << ", started at: "
+	<< Timer::now() << endl;
     static int service_count=0;
     static pthread_mutex_t my_lock = PTHREAD_MUTEX_INITIALIZER;
     pthread_mutex_lock(&my_lock);
@@ -350,72 +371,85 @@ namespace TimblServer {
   }
 
   void HttpServerBase::sendReject( ostream& os ) const {
+    /// send HTTP message that we are too busy
     os << "Status:503 Maximum number of connections exceeded.\n" << endl;
   }
 
   // ***** This is the routine that is executed from a new HTTP thread *******
   void HttpServerBase::socketChild( childArgs *args ){
+    /// run a HTTP child in a new thread
+    /*!
+      \param args the arguments to use.
+
+      this function sets the connected streams to Non-Blocking and then calls
+      serverBase::socketChild
+    */
     args->socket()->setNonBlocking();
     ServerBase::socketChild( args );
   }
 
   int ServerBase::Run(){
-    LOG << "Starting a " << serverProtocol
-	<< " server on port " << serverPort << endl;
-    if ( !pidFile.empty() ){
+    /// run a Server. Must be configured before.
+    LOG << "Starting a " << _protocol
+	<< " server on port " << _server_port << endl;
+    if ( !_pid_file.empty() ){
       // check validity of pidfile
-      if ( doDaemon && pidFile[0] != '/' ) // make sure the path is absolute
-	pidFile = '/' + pidFile;
-      unlink( pidFile.c_str() ) ;
-      ofstream pid_file( pidFile ) ;
+      if ( _do_daemon && _pid_file[0] != '/' ) {
+	// make sure the path is absolute
+	_pid_file = '/' + _pid_file;
+      }
+      unlink( _pid_file.c_str() );
+      ofstream pid_file( _pid_file );
       if ( !pid_file ){
-	LOG<< "unable to create pidfile:"<< pidFile << endl;
+	LOG<< "unable to create pidfile:"<< _pid_file << endl;
 	LOG<< "not Started" << endl;
 	return EXIT_FAILURE;
       }
     }
     ostream *logS = 0;
-    if ( !logFile.empty() ){
-      if ( doDaemon && logFile[0] != '/' ) // make sure the path is absolute
-	logFile = '/' + logFile;
-      logS = new ofstream( logFile );
+    if ( !_log_file.empty() ){
+      if ( _do_daemon && _log_file[0] != '/' ) {
+	// make sure the path is absolute
+	_log_file = '/' + _log_file;
+      }
+      logS = new ofstream( _log_file );
       if ( logS && logS->good() ){
-	LOG << "switching logging to file " << logFile << endl;
-	myLog.associate( *logS );
+	LOG << "switching logging to file " << _log_file << endl;
+	_my_log.associate( *logS );
 	LOG  << "Started logging " << endl;
 	LOG  << "debugging is " << (doDebug()?"on":"off") << endl;
       }
       else {
 	delete logS;
-	LOG << "unable to create logfile: " << logFile << endl;
+	LOG << "unable to create logfile: " << _log_file << endl;
 	LOG << "not started" << endl;
 	return EXIT_FAILURE;
       }
     }
 
     int start = 1;
-    if ( doDaemon ){
+    if ( _do_daemon ){
       LOG << "running as a dæmon" << endl;
       signal( SIGCHLD, AfterDaemonFun );
-      start = daemonize( 0, logFile.empty() );
+      start = daemonize( 0, _log_file.empty() );
     }
     if ( start < 0 ){
       cerr << "failed to daemonize error= " << strerror(errno) << endl;
       return EXIT_FAILURE;
     };
-    if ( !pidFile.empty() ){
+    if ( !_pid_file.empty() ){
       // we have a liftoff!
       // signal it to the world
-      ofstream pid_file( pidFile ) ;
+      ofstream pid_file( _pid_file ) ;
       if ( !pid_file ){
-	LOG << "unable to create pidfile:"<< pidFile << endl;
+	LOG << "unable to create pidfile:"<< _pid_file << endl;
 	LOG << "server NOT Started" << endl;
 	return EXIT_FAILURE;
       }
       else {
 	pid_t pid = getpid();
 	pid_file << pid << endl;
-	LOG << "wrote PID=" << pid << " to " << pidFile << endl;
+	LOG << "wrote PID=" << pid << " to " << _pid_file << endl;
       }
     }
     // set the attributes
@@ -425,12 +459,13 @@ namespace TimblServer {
       LOG << "Threads: couldn't set attributes" << endl;
       return EXIT_FAILURE;
     }
-    LOG << "Starting Server on port:" << serverPort << endl;
+    LOG << "Now running a " << _protocol
+	<< " server on port " << _server_port << endl;
 
     pthread_t chld_thr;
 
     Sockets::ServerSocket server;
-    string portString = toString<int>(serverPort);
+    string portString = toString<int>(_server_port);
     if ( !server.connect( portString ) ){
       LOG << "failed to start Server: " << server.getMessage() << endl;
       return EXIT_FAILURE;
@@ -450,7 +485,7 @@ namespace TimblServer {
     sigaction( SIGTERM, &act, NULL );
     while( keepGoing ){ // waiting for connections loop
       signal( SIGPIPE, SIG_IGN );
-      Sockets::ServerSocket *newSocket = new Sockets::ServerSocket();
+      Sockets::ClientSocket *newSocket = new Sockets::ClientSocket();
       if ( !server.accept( *newSocket ) ){
 	cerr << "accept failed: " + server.getMessage() << endl;
 	delete newSocket;
