@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2018
+  Copyright (c) 2006 - 2020
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -40,16 +40,32 @@ namespace TiCC {
   using namespace icu;
 
   UnicodeString UnicodeFromEnc( const string& s, const string& enc ){
+    /// convert a character buffer in some encoding to an UnicodeString
+    /*!
+      \param s the string to interpret as a character buffer
+      \param enc the encoding to use
+      \return an UnicodeString object
+    */
     return UnicodeString( s.c_str(), s.length(), enc.c_str() );
   }
 
   string UnicodeToUTF8( const UnicodeString& s ){
+    /// convert an UnicodeString to an UTF-8 string
+    /*!
+      \param s the UnicodeString to convert
+      \return an UTF-8 encoded string
+    */
     string result;
     s.toUTF8String(result);
     return result;
   }
 
   UnicodeNormalizer::UnicodeNormalizer( const string& enc ): _normalizer(0) {
+    /// create an UnicodeNormalizer object
+    /*!
+      \param enc a string describing the wanted normaliztion.
+      valid values are: NFC (the default), NFD, NFKC, NFKD
+    */
     string mode = enc;
     if ( mode.empty() ){
       mode = "NFC";
@@ -58,10 +74,16 @@ namespace TiCC {
   }
 
   UnicodeNormalizer::~UnicodeNormalizer(){
+    /// destroy the UnicodeNormalizer
     // NEVER EVER delete _normalizer!
   }
 
   const string UnicodeNormalizer::setMode( const string& enc ){
+    /// set the desired normalizer mode
+    /*!
+      \param enc the new mode to set
+      \return the previous mode
+    */
     if ( enc == mode
 	 || (enc.empty() && mode == "NFC") ){
       return mode;
@@ -70,16 +92,21 @@ namespace TiCC {
       // NEVER EVER delete _normalizer! it is static
       UErrorCode err = U_ZERO_ERROR;
       if ( enc == ""
-	   || enc == "NFC" )
+	   || enc == "NFC" ){
 	_normalizer = Normalizer2::getNFCInstance( err );
-      else if ( enc == "NONE" )
+      }
+      else if ( enc == "NONE" ){
 	_normalizer = 0;
-      else if ( enc == "NFD" )
+      }
+      else if ( enc == "NFD" ){
 	_normalizer = Normalizer2::getNFDInstance( err );
-      else if ( enc == "NFKC" )
+      }
+      else if ( enc == "NFKC" ){
 	_normalizer = Normalizer2::getNFKCInstance( err );
-      else if ( enc == "NFKD" )
+      }
+      else if ( enc == "NFKD" ){
 	_normalizer = Normalizer2::getNFKDInstance( err );
+      }
       else {
 	throw logic_error( "invalid normalization mode: " + enc );
       }
@@ -93,6 +120,11 @@ namespace TiCC {
   }
 
   UnicodeString UnicodeNormalizer::normalize( const UnicodeString& us ){
+    /// normalize a UnicodeString to the current mode
+    /*!
+      \param us the UnicodeString to normalize
+      \return the UnicodeString in the correct normalization
+    */
     if ( _normalizer == 0 ){
       return us;
     }
@@ -106,14 +138,16 @@ namespace TiCC {
     }
   }
 
+  /// @cond HIDDEN
   class uRegexError: public invalid_argument {
   public:
     explicit uRegexError( const string& s ): invalid_argument( "Invalid regular expression: " + s ){};
     explicit uRegexError( const UnicodeString& us ): invalid_argument( "Invalid regular expression: " + UnicodeToUTF8(us) ){};
   };
+  /// @endcond
 
-
-  UnicodeString UnicodeRegexMatcher::Pattern() const{
+  UnicodeString UnicodeRegexMatcher::Pattern() const {
+    /// return the current Regex pattern
     return pattern->pattern();
   }
 
@@ -121,6 +155,11 @@ namespace TiCC {
 					    const UnicodeString& name ):
     _name(name), _debug(false)
   {
+    /// create a RegexMatcher object
+    /*!
+      \param pat The pattern to use
+      \param name a name we give to this RegexMatcher (for error messages)
+    */
     matcher = NULL;
     UErrorCode u_stat = U_ZERO_ERROR;
     UParseError errorInfo;
@@ -148,6 +187,7 @@ namespace TiCC {
   }
 
   UnicodeRegexMatcher::~UnicodeRegexMatcher(){
+    /// destroy a RegexMatcher
     delete pattern;
     delete matcher;
   }
@@ -155,6 +195,15 @@ namespace TiCC {
   bool UnicodeRegexMatcher::match_all( const UnicodeString& line,
 				       UnicodeString& pre,
 				       UnicodeString& post ){
+    /// apply the RegexMatcher on an Unicode line
+    /*!
+      \param line the UnicodeString to analyze
+      \param pre the part of the line BEFORE the match, may be ""
+      \param post the part of the line AFTER the match, may be ""
+      \return true when there was some match found
+
+      if match_all returns true, you need to call get_match() to get results
+    */
     UErrorCode u_stat = U_ZERO_ERROR;
     pre = "";
     post = "";
@@ -254,7 +303,7 @@ namespace TiCC {
 	    if ( _debug ){
 	      cerr << "start = " << start << endl;
 	    }
-	    if (!U_FAILURE(u_stat)){
+	    if ( !U_FAILURE(u_stat) ){
 	      if ( start < 0 ){
 		continue;
 	      }
@@ -271,7 +320,7 @@ namespace TiCC {
 	    if ( _debug ){
 	      cerr << "end = " << end << endl;
 	    }
-	    if (!U_FAILURE(u_stat)){
+	    if ( !U_FAILURE(u_stat) ){
 	      results.push_back( UnicodeString( line, start, end - start ) );
 	      if ( _debug ){
 		cerr << "added result " << results.back() << endl;
@@ -295,37 +344,55 @@ namespace TiCC {
   }
 
   const UnicodeString UnicodeRegexMatcher::get_match( unsigned int n ) const{
-    if ( n < results.size() )
+    /// get one match from the RegexMatcher
+    /*!
+      \param n the index of the match
+      \return the match result as a UnicodeString. Returns "" when n is out
+      of range.
+    */
+    if ( n < results.size() ){
       return results[n];
-    else
-      return "";
+    }
+    return "";
   }
 
   int UnicodeRegexMatcher::NumOfMatches() const {
-    if ( results.size() > 0 )
+    /// give the number of matches found.
+    if ( results.size() > 0 ){
       return results.size()-1;
-    else
-      return 0;
+    }
+    return 0;
   }
 
   int UnicodeRegexMatcher::split( const UnicodeString& us,
 				  vector<UnicodeString>& result ){
+    /// split a UnicodeString using the stored pattern
+    /*!
+      \param us the UnicodeString to split
+      \param result a vector with the splitted parts
+      \return the number os elements in the result
+    */
     result.clear();
     const int maxWords = 256;
     UnicodeString words[maxWords];
     UErrorCode status = U_ZERO_ERROR;
     int numWords = matcher->split( us, words, maxWords, status );
-    for ( int i = 0; i < numWords; ++i )
+    for ( int i = 0; i < numWords; ++i ){
       result.push_back( words[i] );
+    }
     return numWords;
   }
 
-  UniFilter::UniFilter(): _trans(0) {}
+  UniFilter::UniFilter(): _trans(0) {
+    /// create a Unicode Filter object
+  }
   UniFilter::~UniFilter(){
+    /// destroy a Unicode Filter object
     delete _trans;
   }
 
   UnicodeString UniFilter::get_rules() const {
+    /// extract the current rules from the Unicode Filter
     UnicodeString result;
     if ( !_trans ){
       throw runtime_error( "UniFilter::getRules(), filter not initialized." );
@@ -337,6 +404,12 @@ namespace TiCC {
 
   bool UniFilter::init( const UnicodeString& rules,
 			const UnicodeString& name ){
+    /// initialize a Unicode Filter
+    /*!
+      \param rules a Unicode string with filter rules
+      \param name a name for the filter (used for error messages)
+      \return true on succes, will throw on error
+    */
     if ( _trans ){
       throw logic_error( "UniFilter::init():, filter already initialized." );
     }
@@ -357,9 +430,19 @@ namespace TiCC {
   }
 
   UnicodeString to_icu_rule( const UnicodeString& line ){
-    // a line can be an ICU Transcriptor rule " ß > sz ;"
-    // OR a simple mentioning of a symbol to be replaced " ss sz" (old_style)
-    // we try to covert old style to a ICU rule. (always only 1)
+    /// convert an ICU Transcriptor rule or a trivial replacement into
+    /// an ICU rule
+    /*!
+      \param line the rule to convert
+      \return an Unicode representation of the rule
+
+      A rule can be an ICU Transcriptor rule like " ß > sz ;"
+      OR a simple mentioning of a symbol to be replaced " ss sz" (old_style)
+
+      The old_style variants are converted to a ICU rule. (always only just 1)
+
+      otherwise the input is just returned 'as is'
+    */
     bool old_style = line.indexOf( '>' ) == -1;
     if ( old_style ){
       UnicodeString result;
@@ -368,8 +451,7 @@ namespace TiCC {
 	if ( line[i] == '`' || line[i] == '\'' || line[i] == '"' ){
 	  result += '\\';
 	}
-	else if ( old_style
-		  && (line[i] == ' ' || line[i] == '\t' )
+	else if ( (line[i] == ' ' || line[i] == '\t' )
 		  && !inserted ){
 	  // OLD style: replace first space by a '>' symbol.
 	  inserted = true;
@@ -392,6 +474,12 @@ namespace TiCC {
 
   bool UniFilter::fill( const string& filename,
 			const string& label ){
+    /// fill a Unicode Filter from a file
+    /*!
+      \param filename the file to read
+      \param label a label for the filter
+      \return true on succes, will throw on erroe
+    */
     ifstream is( filename );
     if ( !is ){
       throw runtime_error( "UniFilter::fill(), unable te open rules file: '"
@@ -407,6 +495,11 @@ namespace TiCC {
   }
 
   UnicodeString UniFilter::filter( const UnicodeString& line ){
+    /// apply the Unicode Filter on a Unicode line
+    /*!
+      \param line the inputline
+      \return the resulting filtered line
+    */
     if ( !_trans ){
       //      throw logic_error( "UniFilter not initialized." );
       return line;
@@ -419,6 +512,10 @@ namespace TiCC {
   }
 
   bool UniFilter::add( const UnicodeString& in ){
+    /// add an extra rule to the Unicode Filter
+    /*!
+      \param in a rule to add
+    */
     //
     // TODO: cache multiple add's and only (re-)init the transliterator
     //       once. On first use of the filter() method.
@@ -442,16 +539,26 @@ namespace TiCC {
   }
 
   bool UniFilter::add( const string& line ){
+    /// add an extra rule to the Unicode Filter
+    /*!
+      \param line a UTF-8 encoded rule
+    */
     UnicodeString uline = UnicodeFromUTF8( line );
     return add( uline );
   }
 
   ostream& operator<<( ostream& os, const UniFilter& uf ){
+    /// output the current Rules to a stream
     os << uf.get_rules();
     return os;
   }
 
   UnicodeString filter_diacritics( const UnicodeString& in ) {
+    /// filter ALL diacritics from an UnicodeString
+    /*!
+      \param in the UnicodeString to filter from
+      \return an UnicodeString with all diacrytics removed
+    */
     static Transliterator *trans = 0;
     if ( trans == 0 ){
       UErrorCode stat = U_ZERO_ERROR;
@@ -470,10 +577,17 @@ namespace TiCC {
   vector<UnicodeString> split_at( const UnicodeString& src,
 				  const UnicodeString& sep,
 				  size_t max ){
-    // split a string into substrings, using sep as separator
-    // silently skip empty entries (e.g. when two or more separators co-incide)
-    // if max > 0, limit the size off the result to max,
-    // leaving the remainder in the last part of the result
+    /// split an UnicodeString
+    /*!
+      \param src the UnicodeString to split
+      \param sep the separator to split at
+      \param max limit the size off the result to max, when max > 0
+      leaving the remainder in the last part of the result
+      \return a vector with the splitted parts
+
+      \note this function skips empty entries (e.g. when two or more separators
+      co-incide)
+    */
     if ( sep.isEmpty() ){
       throw runtime_error( "TiCC::split_at(): separator is empty!" );
     }
@@ -508,6 +622,13 @@ namespace TiCC {
   int find_first_of( const UnicodeString& src,
 		     const UnicodeString& seps,
 		     int pos ){
+    /// find the first occurrence of one of the seps in a string
+    /*!
+      \param src the string to search
+      \param seps a list of separator characters
+      \param pos start position for the search
+      \return the position found, or -1 when not present
+    */
     int result = src.length()+10;
     for ( int i=0; i < seps.length(); ++i ){
       int p = src.indexOf( seps[i], pos );
@@ -524,11 +645,17 @@ namespace TiCC {
   vector<UnicodeString> split_at_first_of( const UnicodeString& src,
 					   const UnicodeString& seps,
 					   size_t max ){
-    // split a string into substrings, using the characters in seps
-    // as possible separators
-    // silently skip empty entries (e.g. when two or more separators co-incide)
-    // if max > 0, limit the size of the result to max,
-    // leaving the remainder in the last part of the result
+    /// split an UnicodeString
+    /*!
+      \param src the UnicodeString to split
+      \param seps a list of separator characters
+      \param max limit the size off the result to max, when max > 0
+      leaving the remainder in the last part of the result
+      \return a vector with the splitted parts
+
+      \note this function skips empty entries (e.g. when two or more separators
+      co-incide)
+    */
     if ( seps.isEmpty() ){
       throw runtime_error( "TiCC::split_at_first_of(): separators are empty!" );
     }
@@ -562,19 +689,29 @@ namespace TiCC {
 
   vector<UnicodeString> split( const UnicodeString& src,
 			       size_t max ){
+    /// split an UnicodeString at whitespace
+    /*!
+      \param src the UnicodeString to split
+      \param max limit the size off the result to max, when max > 0
+      leaving the remainder in the last part of the result
+      \return a vector with the splitted parts
+
+      \note this function skips empty entries (e.g. when two or more separators
+      co-incide)
+    */
     static UnicodeString spaces = TiCC::UnicodeFromUTF8( " \r\t\n" );
     return split_at_first_of( src, spaces, max );
   }
 
   string utf8_lowercase( const string& in ){
-    // Unicode safe version
+    /// convert an UTF-8 string to lowercase
     UnicodeString us = TiCC::UnicodeFromUTF8( in );
     us.toLower();
     return TiCC::UnicodeToUTF8( us );
   }
 
   string utf8_uppercase( const string& in ){
-    // Unicode safe version
+    /// convert an UTF-8 string to uppercase
     UnicodeString us = TiCC::UnicodeFromUTF8( in );
     us.toUpper();
     return TiCC::UnicodeToUTF8( us );
